#include "main.h"
#include "stm32f429i_discovery_lcd.h"

#include <stdio.h>

#include "tn.h"
#include "tn_port.h"
#include "tn_utils.h"

#define  LINE_10_BUSY 1   // 0b0000 0001
#define  LINE_11_BUSY 2   // 0b0000 0010

//----------- Tasks ----------------------------------------------------------

#define  TASK_1_PRIORITY      4
#define  TASK_2_PRIORITY      4
#define  TASK_3_PRIORITY      5
#define  TASK_4_PRIORITY      5
#define  TASK_5_PRIORITY      5
#define  TASK_6_PRIORITY      2
#define  TASK_7_PRIORITY      2
#define  TASK_8_PRIORITY      1
#define  TASK_9_PRIORITY      4
#define  TASK_10_PRIORITY     4

#define  TASK_1_STK_SIZE      1024
#define  TASK_2_STK_SIZE      1024
#define  TASK_3_STK_SIZE      1024
#define  TASK_4_STK_SIZE      1024
#define  TASK_5_STK_SIZE      1024
#define  TASK_6_STK_SIZE      1024
#define  TASK_7_STK_SIZE      1024
#define  TASK_8_STK_SIZE      1024
#define  TASK_9_STK_SIZE      1024
#define  TASK_10_STK_SIZE     1024


#if defined (__ICCARM__)    // IAR ARM
#pragma data_alignment=8
#endif
align_attr_start unsigned int task_1_stack[TASK_1_STK_SIZE] align_attr_end;

#if defined (__ICCARM__)    // IAR ARM
#pragma data_alignment=8
#endif
align_attr_start unsigned int task_2_stack[TASK_2_STK_SIZE]  align_attr_end;

#if defined (__ICCARM__)    // IAR ARM
#pragma data_alignment=8
#endif
align_attr_start unsigned int task_3_stack[TASK_3_STK_SIZE]  align_attr_end;

#if defined (__ICCARM__)    // IAR ARM
#pragma data_alignment=8
#endif
align_attr_start unsigned int task_4_stack[TASK_4_STK_SIZE]  align_attr_end;

#if defined (__ICCARM__)    // IAR ARM
#pragma data_alignment=8
#endif
align_attr_start unsigned int task_5_stack[TASK_5_STK_SIZE]  align_attr_end;

#if defined (__ICCARM__)    // IAR ARM
#pragma data_alignment=8
#endif
align_attr_start unsigned int task_6_stack[TASK_6_STK_SIZE]  align_attr_end;

#if defined (__ICCARM__)    // IAR ARM
#pragma data_alignment=8
#endif
align_attr_start unsigned int task_7_stack[TASK_7_STK_SIZE]  align_attr_end;

#if defined (__ICCARM__)    // IAR ARM
#pragma data_alignment=8
#endif
align_attr_start unsigned int task_8_stack[TASK_8_STK_SIZE]  align_attr_end;

#if defined (__ICCARM__)    // IAR ARM
#pragma data_alignment=8
#endif
align_attr_start unsigned int task_9_stack[TASK_9_STK_SIZE]  align_attr_end;

#if defined (__ICCARM__)    // IAR ARM
#pragma data_alignment=8
#endif
align_attr_start unsigned int task_10_stack[TASK_10_STK_SIZE]  align_attr_end;

TN_TCB  task_1;
TN_TCB  task_2;
TN_TCB  task_3;
TN_TCB  task_4;
TN_TCB  task_5;
TN_TCB  task_6;
TN_TCB  task_7;
TN_TCB  task_8;
TN_TCB  task_9;
TN_TCB  task_10;

void task_1_func(void * par);
void task_2_func(void * par);
void task_3_func(void * par);
void task_4_func(void * par);
void task_5_func(void * par);
void task_6_func(void * par);
void task_7_func(void * par);
void task_8_func(void * par);
void task_9_func(void * par);
void task_10_func(void * par);


#define  QUEUE_1_SIZE      4
void    *queue_1_mem[QUEUE_1_SIZE];
TN_DQUE  queue_1;
TN_MUTEX mutex_1;
TN_SEM   sem_1;
TN_EVENT event_1;

void  tn_app_init(void);

int main(void)
{  
    SCB->SHCSR |= SCB_SHCSR_BUSFAULTENA_Msk;
    SCB->SHCSR |= SCB_SHCSR_MEMFAULTENA_Msk;
    SCB->SHCSR |= SCB_SHCSR_USGFAULTENA_Msk;
    
    tn_arm_disable_interrupts();
    
    SystemCoreClockUpdate();
    SysTick_Config(SystemCoreClock/1000);

    LCD_Init();  
    LCD_LayerInit();

    LTDC_Cmd(ENABLE);
 
    LCD_SetLayer(LCD_FOREGROUND_LAYER);
    LCD_Clear(LCD_COLOR_BLACK);
    LCD_SetBackColor(LCD_COLOR_BLACK);
    LCD_SetTextColor(LCD_COLOR_WHITE);
    
    LCD_SetFont(&Font8x12);
    
    LCD_DisplayStringLine(LCD_LINE_0, (uint8_t *)" TEST RTOS ");
    LCD_DisplayStringLine(LCD_LINE_1, (uint8_t *)" TNKenel ");
    LCD_DisplayStringLine(LCD_LINE_3, (uint8_t *)" The BADEMBED generation ");
    
    GPIO_InitTypeDef GPIO_InitStructure;

    //  
    RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA, ENABLE);

    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN; 
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL; 
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0; 
    GPIO_Init(GPIOA, &GPIO_InitStructure);
    
    tn_start_system();
    while(1);
}

void  tn_app_init(void)
{
   //--- Task 1

   task_1.id_task = 0;
   tn_task_create(&task_1,               //-- task TCB
                 task_1_func,            //-- task function
                 TASK_1_PRIORITY,        //-- task priority
                 &(task_1_stack          //-- task stack first addr in memory
                    [TASK_1_STK_SIZE-1]),
                 TASK_1_STK_SIZE,        //-- task stack size (in int,not bytes)
                 NULL,                          //-- task function parameter
                 TN_TASK_START_ON_CREATION);    //-- Creation option

   //--- Task 2

   task_2.id_task = 0;
   tn_task_create(&task_2,                      //-- task TCB
                 task_2_func,                   //-- task function
                 TASK_2_PRIORITY,               //-- task priority
                 &(task_2_stack                 //-- task stack first addr in memory
                    [TASK_2_STK_SIZE-1]),
                 TASK_2_STK_SIZE,               //-- task stack size (in int,not bytes)
                 NULL,                           //-- task function parameter
                 TN_TASK_START_ON_CREATION);     //-- Creation option
   
   task_3.id_task = 0;
   tn_task_create(&task_3,                      //-- task TCB
                 task_3_func,                   //-- task function
                 TASK_3_PRIORITY,               //-- task priority
                 &(task_3_stack                 //-- task stack first addr in memory
                    [TASK_3_STK_SIZE-1]),
                 TASK_3_STK_SIZE,               //-- task stack size (in int,not bytes)
                 NULL,                           //-- task function parameter
                 TN_TASK_START_ON_CREATION);     //-- Creation option
   
   task_4.id_task = 0;
   tn_task_create(&task_4,                      //-- task TCB
                 task_4_func,                   //-- task function
                 TASK_4_PRIORITY,               //-- task priority
                 &(task_4_stack                 //-- task stack first addr in memory
                    [TASK_4_STK_SIZE-1]),
                 TASK_4_STK_SIZE,               //-- task stack size (in int,not bytes)
                 NULL,                           //-- task function parameter
                 TN_TASK_START_ON_CREATION);     //-- Creation option
   
   task_5.id_task = 0;
   tn_task_create(&task_5,                      //-- task TCB
                 task_5_func,                   //-- task function
                 TASK_5_PRIORITY,               //-- task priority
                 &(task_5_stack                 //-- task stack first addr in memory
                    [TASK_5_STK_SIZE-1]),
                 TASK_5_STK_SIZE,               //-- task stack size (in int,not bytes)
                 NULL,                           //-- task function parameter
                 TN_TASK_START_ON_CREATION);     //-- Creation option
   
   task_6.id_task = 0;
   tn_task_create(&task_6,                      //-- task TCB
                 task_6_func,                   //-- task function
                 TASK_6_PRIORITY,               //-- task priority
                 &(task_6_stack                 //-- task stack first addr in memory
                    [TASK_6_STK_SIZE-1]),
                 TASK_6_STK_SIZE,               //-- task stack size (in int,not bytes)
                 NULL,                           //-- task function parameter
                 TN_TASK_START_ON_CREATION);     //-- Creation option
   
   task_7.id_task = 0;
   tn_task_create(&task_7,                      //-- task TCB
                 task_7_func,                   //-- task function
                 TASK_7_PRIORITY,               //-- task priority
                 &(task_7_stack                 //-- task stack first addr in memory
                    [TASK_7_STK_SIZE-1]),
                 TASK_7_STK_SIZE,               //-- task stack size (in int,not bytes)
                 NULL,                           //-- task function parameter
                 TN_TASK_START_ON_CREATION);     //-- Creation option
   
   task_8.id_task = 0;
   tn_task_create(&task_8,                      //-- task TCB
                 task_8_func,                   //-- task function
                 TASK_8_PRIORITY,               //-- task priority
                 &(task_8_stack                 //-- task stack first addr in memory
                    [TASK_8_STK_SIZE-1]),
                 TASK_8_STK_SIZE,               //-- task stack size (in int,not bytes)
                 NULL,                           //-- task function parameter
                 TN_TASK_START_ON_CREATION);     //-- Creation option
   
   task_9.id_task = 0;
   tn_task_create(&task_9,                      //-- task TCB
                 task_9_func,                   //-- task function
                 TASK_9_PRIORITY,               //-- task priority
                 &(task_9_stack                 //-- task stack first addr in memory
                    [TASK_9_STK_SIZE-1]),
                 TASK_9_STK_SIZE,               //-- task stack size (in int,not bytes)
                 NULL,                           //-- task function parameter
                 TN_TASK_START_ON_CREATION);     //-- Creation option
   
   task_10.id_task = 0;
   tn_task_create(&task_10,                      //-- task TCB
                 task_10_func,                   //-- task function
                 TASK_10_PRIORITY,               //-- task priority
                 &(task_10_stack                 //-- task stack first addr in memory
                    [TASK_10_STK_SIZE-1]),
                 TASK_10_STK_SIZE,               //-- task stack size (in int,not bytes)
                 NULL,                           //-- task function parameter
                 TN_TASK_START_ON_CREATION);     //-- Creation option
   
   mutex_1.id_mutex = 0;
   tn_mutex_create(&mutex_1,
                   TN_MUTEX_ATTR_INHERIT,       //   
                   0);      //    -   
   
   sem_1.id_sem = 0;
   tn_sem_create(&sem_1,
                 2,   //-- Start value
                 2);  //-- Max value
   
   event_1.id_event = 0;
   tn_event_create(&event_1,
                   TN_EVENT_ATTR_SINGLE | TN_EVENT_ATTR_CLR, // Eventflag attribute
                   0);       // Initial value of the eventflag bit pattern
   
   queue_1.id_dque = 0;
   tn_queue_create(&queue_1,       //-- Ptr to already existing TN_DQUE
                   &queue_1_mem[0],  //-- Ptr to already existing array of void* 
                   QUEUE_1_SIZE); //-- Capacity of data queue(num entries)

}

void task_1_func(void * par)
{
    while(1)
    {
        tn_mutex_lock(&mutex_1, TN_WAIT_INFINITE);
        
        LCD_ClearLine(LCD_LINE_6);
        LCD_ClearLine(LCD_LINE_7);
        LCD_DisplayStringLine(LCD_LINE_6, (uint8_t *)" TASK 1 ");
        
        tn_task_sleep(2000);
        
        tn_mutex_unlock(&mutex_1);
    }
}

void task_2_func(void * par)
{
    while(1)
    {
        tn_mutex_lock(&mutex_1, TN_WAIT_INFINITE);
        
        LCD_ClearLine(LCD_LINE_6);
        LCD_ClearLine(LCD_LINE_7);
        LCD_DisplayStringLine(LCD_LINE_7, (uint8_t *)" TASK 2 ");
        
        tn_task_sleep(3000);
        
        tn_mutex_unlock(&mutex_1);
    }
}

uint8_t f_line_busy = 0;

void task_3_func(void * par)
{   
    while(1)
    {
        tn_sem_acquire(&sem_1, TN_WAIT_INFINITE);
        
        if((f_line_busy & LINE_10_BUSY) == 0)
        {
            f_line_busy |= LINE_10_BUSY;
            
            LCD_ClearLine(LCD_LINE_10);
            LCD_DisplayStringLine(LCD_LINE_10, (uint8_t *)" TASK 3 ");
            
            tn_task_sleep(2000);
            f_line_busy &= ~LINE_10_BUSY;
        }
        else if((f_line_busy & LINE_11_BUSY) == 0)
        {
            f_line_busy |= LINE_11_BUSY;
            
            LCD_ClearLine(LCD_LINE_11);
            LCD_DisplayStringLine(LCD_LINE_11, (uint8_t *)" TASK 3 ");
            
            tn_task_sleep(2000);
            f_line_busy &= ~LINE_11_BUSY;
        }
        else
        {
            while(1)
            {
                LCD_ClearLine(LCD_LINE_11);
                LCD_DisplayStringLine(LCD_LINE_11, (uint8_t *)" ERROR ");
            }
        }
        
        tn_sem_signal(&sem_1);
    }
}

void task_4_func(void * par)
{   
    while(1)
    {
        tn_sem_acquire(&sem_1, TN_WAIT_INFINITE);
        
        if((f_line_busy & LINE_10_BUSY) == 0)
        {
            f_line_busy |= LINE_10_BUSY;
            
            LCD_ClearLine(LCD_LINE_10);
            LCD_DisplayStringLine(LCD_LINE_10, (uint8_t *)" TASK 4 ");
            
            tn_task_sleep(2000);
            f_line_busy &= ~LINE_10_BUSY;
        }
        else if((f_line_busy & LINE_11_BUSY) == 0)
        {
            f_line_busy |= LINE_11_BUSY;
            
            LCD_ClearLine(LCD_LINE_11);
            LCD_DisplayStringLine(LCD_LINE_11, (uint8_t *)" TASK 4 ");
            
            tn_task_sleep(2000);
            f_line_busy &= ~LINE_11_BUSY;
        }
        else
        {
            while(1)
            {
                LCD_ClearLine(LCD_LINE_11);
                LCD_DisplayStringLine(LCD_LINE_11, (uint8_t *)" ERROR ");
            }
        }
        
        tn_sem_signal(&sem_1);
    }
}

void task_5_func(void * par)
{   
    while(1)
    {
        tn_sem_acquire(&sem_1, TN_WAIT_INFINITE);
        
        if((f_line_busy & LINE_10_BUSY) == 0)
        {
            f_line_busy |= LINE_10_BUSY;
            
            LCD_ClearLine(LCD_LINE_10);
            LCD_DisplayStringLine(LCD_LINE_10, (uint8_t *)" TASK 5 ");
            
            tn_task_sleep(2000);
            f_line_busy &= ~LINE_10_BUSY;
        }
        else if((f_line_busy & LINE_11_BUSY) == 0)
        {
            f_line_busy |= LINE_11_BUSY;
            
            LCD_ClearLine(LCD_LINE_11);
            LCD_DisplayStringLine(LCD_LINE_11, (uint8_t *)" TASK 5 ");
            
            tn_task_sleep(2000);
            f_line_busy &= ~LINE_11_BUSY;
        }
        else
        {
            while(1)
            {
                LCD_ClearLine(LCD_LINE_11);
                LCD_DisplayStringLine(LCD_LINE_11, (uint8_t *)" ERROR ");
            }           
        }
        
        tn_sem_signal(&sem_1);
    }
}

void task_6_func(void * par)
{
    uint8_t f_push_btn;
    
    while(1)
    {
        f_push_btn = GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_0);
        if(f_push_btn == 1)
        {
            tn_event_set(&event_1, 1);
            tn_task_sleep(3000);
        }
        tn_task_sleep(50);
    }
}

void task_7_func(void * par)
{
    while(1)
    {
        tn_task_sleep(8000);
        tn_event_set(&event_1, 2);
    }
}

void task_8_func(void * par)
{
    unsigned int f_event;
    
    while(1)
    {
        tn_event_wait(&event_1,
                    0x01 + 0x02,        //  event
                    TN_EVENT_WCOND_OR,  //  1 event
                    &f_event,           //  event
                    TN_WAIT_INFINITE);  //  
        
        if(f_event & 0x01)
        {
            LCD_ClearLine(LCD_LINE_14);
            LCD_DisplayStringLine(LCD_LINE_14, (uint8_t *)" PUSH BTN EVT");
        }
        else if(f_event & 0x02)
        {
            LCD_ClearLine(LCD_LINE_14);
            LCD_DisplayStringLine(LCD_LINE_14, (uint8_t *)" TIMEOUT EVT ");
        }
        else
        {
            while(1)
            {
                LCD_ClearLine(LCD_LINE_14);
                LCD_DisplayStringLine(LCD_LINE_14, (uint8_t *)" ERROR ");
            }
        }
    }
}

void task_9_func(void * par)
{
    unsigned int tx_data = 0;
    unsigned int cnt = 0;
    
    while(1)
    {
        tx_data = cnt;
        tn_queue_send(&queue_1, (void*)&tx_data, TN_WAIT_INFINITE);
        tn_task_sleep(1000);
        cnt++;
    }
}

void task_10_func(void * par)
{
    int rc;
    unsigned int *rx_data;
    char out_buf[20];
    
    while(1)
    {
        rc = tn_queue_receive(&queue_1, (void **)&rx_data, TN_WAIT_INFINITE);
        if(rc == TERR_NO_ERR)
        {
            sprintf(out_buf, " count = %u", *rx_data); 
            LCD_ClearLine(LCD_LINE_17);
            LCD_DisplayStringLine(LCD_LINE_17, (uint8_t *)out_buf);
        } 
        else
        {
            LCD_ClearLine(LCD_LINE_17);
            LCD_DisplayStringLine(LCD_LINE_17, (uint8_t *)" ERROR ");
        }
    }
}

